aboutsummaryrefslogtreecommitdiff
diff options
context:
space:
mode:
-rw-r--r--inc/log.h11
-rw-r--r--options.ggo1
-rw-r--r--src/build.c9
-rw-r--r--src/free.c3
-rw-r--r--src/log.c36
-rw-r--r--src/main.c33
-rw-r--r--src/parse.c9
-rw-r--r--src/print.c5
-rw-r--r--src/term.c7
-rw-r--r--src/tree.c21
10 files changed, 102 insertions, 33 deletions
diff --git a/inc/log.h b/inc/log.h
new file mode 100644
index 0000000..8d33658
--- /dev/null
+++ b/inc/log.h
@@ -0,0 +1,11 @@
+// Copyright (c) 2023, Marvin Borner <dev@marvinborner.de>
+// SPDX-License-Identifier: MIT
+
+#ifndef BLOC_LOG_H
+#define BLOC_LOG_H
+
+void debug(const char *format, ...);
+void debug_enable(int enable);
+void fatal(const char *format, ...) __attribute__((noreturn));
+
+#endif
diff --git a/options.ggo b/options.ggo
index 10c30e3..11626b3 100644
--- a/options.ggo
+++ b/options.ggo
@@ -4,6 +4,7 @@ purpose "Tool for converting to/from BLC and BLoC"
option "input" i "input file" string required
option "output" o "output file" default="out" string optional
+option "verbose" v "enable debug logging output" flag off
option "from-blc" b "convert from BLC to BLoC" flag off
option "from-bloc" B "convert from BLoC to BLC" flag off
option "dump" d "dump bloc file" dependon="from-bloc" flag off
diff --git a/src/build.c b/src/build.c
index d8a8146..fff5651 100644
--- a/src/build.c
+++ b/src/build.c
@@ -6,6 +6,7 @@
#include <stdio.h>
#include <build.h>
+#include <log.h>
static void write_bit(char val, FILE *file, char *byte, int *bit)
{
@@ -51,7 +52,7 @@ static void rec_write_bblc(struct tree *tree, FILE *file, char *byte, int *bit)
write_bit((ref >> i) & 1, file, byte, bit);
break;
default:
- fprintf(stderr, "invalid type %d\n", tree->type);
+ fatal("invalid type %d\n", tree->type);
}
}
@@ -69,6 +70,7 @@ static void write_bblc(struct tree *tree, FILE *file)
void write_bloc(struct list *table, const char *path)
{
short length = table->val;
+ debug("writing bloc with %ld elements to %s\n", length, path);
FILE *file = fopen(path, "wb");
fwrite(BLOC_IDENTIFIER, BLOC_IDENTIFIER_LENGTH, 1, file);
@@ -103,14 +105,13 @@ static void fprint_bloc_blc(struct term *term, struct bloc_parsed *bloc,
break;
case REF:
if (term->u.ref.index + 1 >= bloc->length)
- fprintf(stderr, "invalid ref index %ld\n",
- term->u.ref.index);
+ fatal("invalid ref index %ld\n", term->u.ref.index);
fprint_bloc_blc(
bloc->entries[bloc->length - term->u.ref.index - 2],
bloc, file);
break;
default:
- fprintf(stderr, "invalid type %d\n", term->type);
+ fatal("invalid type %d\n", term->type);
}
}
diff --git a/src/free.c b/src/free.c
index 1d512fa..ecf1d9f 100644
--- a/src/free.c
+++ b/src/free.c
@@ -5,6 +5,7 @@
#include <stdlib.h>
#include <free.h>
+#include <log.h>
void free_term(struct term *term)
{
@@ -25,7 +26,7 @@ void free_term(struct term *term)
free(term);
break;
default:
- fprintf(stderr, "invalid type %d\n", term->type);
+ fatal("invalid type %d\n", term->type);
}
}
diff --git a/src/log.c b/src/log.c
new file mode 100644
index 0000000..06c3614
--- /dev/null
+++ b/src/log.c
@@ -0,0 +1,36 @@
+// Copyright (c) 2023, Marvin Borner <dev@marvinborner.de>
+// SPDX-License-Identifier: MIT
+
+#include <stdio.h>
+#include <stdarg.h>
+#include <stdlib.h>
+
+#include <log.h>
+
+static int debug_enabled = 0;
+
+void debug(const char *format, ...)
+{
+ if (!debug_enabled)
+ return;
+
+ va_list ap;
+ va_start(ap, format);
+ vfprintf(stderr, format, ap);
+ va_end(ap);
+}
+
+void debug_enable(int enable)
+{
+ debug_enabled = enable;
+}
+
+void fatal(const char *format, ...)
+{
+ va_list ap;
+ va_start(ap, format);
+ vfprintf(stderr, format, ap);
+ va_end(ap);
+
+ abort();
+}
diff --git a/src/main.c b/src/main.c
index f5e7312..b2d86eb 100644
--- a/src/main.c
+++ b/src/main.c
@@ -7,6 +7,7 @@
#include <stdlib.h>
#include <term.h>
+#include <log.h>
#include <print.h>
#include <tree.h>
#include <parse.h>
@@ -19,6 +20,7 @@
#define BUF_SIZE 1024
static char *read_stdin(void)
{
+ debug("reading from stdin\n");
char buffer[BUF_SIZE];
size_t size = 1;
char *string = malloc(sizeof(char) * BUF_SIZE);
@@ -38,20 +40,17 @@ static char *read_stdin(void)
if (ferror(stdin)) {
free(string);
- fprintf(stderr, "Couldn't read from stdin\n");
- return 0;
+ fatal("can't read from stdin\n");
}
return string;
}
static char *read_file(const char *path)
{
+ debug("reading from %s\n", path);
FILE *f = fopen(path, "rb");
- if (!f) {
- fprintf(stderr, "Can't open file %s: %s\n", path,
- strerror(errno));
- return 0;
- }
+ if (!f)
+ fatal("can't open file %s: %s\n", path, strerror(errno));
fseek(f, 0, SEEK_END);
long fsize = ftell(f);
@@ -62,9 +61,8 @@ static char *read_file(const char *path)
fclose(f);
if (ret != 1) {
- fprintf(stderr, "Can't read file %s: %s\n", path,
- strerror(errno));
- return 0;
+ free(string);
+ fatal("can't read file %s: %s\n", path, strerror(errno));
}
string[fsize] = 0;
@@ -77,6 +75,8 @@ int main(int argc, char **argv)
if (cmdline_parser(argc, argv, &args))
exit(1);
+ debug_enable(args.verbose_flag);
+
char *input;
if (args.input_arg[0] == '-') {
input = read_stdin();
@@ -88,16 +88,27 @@ int main(int argc, char **argv)
return 1;
if (args.from_blc_flag && !args.from_bloc_flag) {
+ debug("parsing as blc\n");
+
struct term *parsed = parse_blc(input);
+ debug("parsed blc\n");
+
+ debug("merging duplicates\n");
struct list *table = tree_merge_duplicates(parsed);
+
write_bloc(table, args.output_arg);
+
tree_destroy(table);
free_term(parsed);
free(input);
+
+ debug("done!\n");
return 0;
}
if (args.from_bloc_flag && !args.from_blc_flag) {
+ debug("parsing as bloc\n");
+
struct bloc_parsed *bloc = parse_bloc(input);
if (args.dump_flag)
print_bloc(bloc);
@@ -107,6 +118,6 @@ int main(int argc, char **argv)
return 0;
}
- fprintf(stderr, "invalid options: use --help for information\n");
+ fatal("invalid options: use --help for information\n");
return 1;
}
diff --git a/src/parse.c b/src/parse.c
index 6e08d2e..b6645fd 100644
--- a/src/parse.c
+++ b/src/parse.c
@@ -8,12 +8,13 @@
#include <term.h>
#include <spec.h>
#include <parse.h>
+#include <log.h>
static struct term *rec_blc(const char **term)
{
struct term *res = 0;
if (!**term) {
- fprintf(stderr, "invalid parsing state!\n");
+ fatal("invalid parsing state!\n");
} else if (**term == '0' && *(*term + 1) == '0') {
(*term) += 2;
res = new_term(ABS);
@@ -89,7 +90,7 @@ struct bloc_parsed *parse_bloc(const void *bloc)
const struct bloc_header *header = bloc;
if (memcmp(header->identifier, BLOC_IDENTIFIER,
(size_t)BLOC_IDENTIFIER_LENGTH)) {
- fprintf(stderr, "invalid BLoC identifier!\n");
+ fatal("invalid BLoC identifier!\n");
return 0;
}
@@ -124,7 +125,7 @@ static struct term *rec_bloc(struct term *term, struct bloc_parsed *bloc)
break;
case REF:
if (term->u.ref.index >= bloc->length) {
- fprintf(stderr, "invalid entry reference\n");
+ fatal("invalid entry reference\n");
return 0;
}
memcpy(term,
@@ -132,7 +133,7 @@ static struct term *rec_bloc(struct term *term, struct bloc_parsed *bloc)
sizeof(*term));
break;
default:
- fprintf(stderr, "invalid type %d\n", term->type);
+ fatal("invalid type %d\n", term->type);
return 0;
}
return term;
diff --git a/src/print.c b/src/print.c
index ddbb730..d73f9c5 100644
--- a/src/print.c
+++ b/src/print.c
@@ -4,6 +4,7 @@
#include <stdio.h>
#include <print.h>
+#include <log.h>
void print_bruijn(struct term *term)
{
@@ -27,7 +28,7 @@ void print_bruijn(struct term *term)
printf("<%ld>", term->u.ref.index);
break;
default:
- fprintf(stderr, "Invalid type %d\n", term->type);
+ fatal("invalid type %d\n", term->type);
}
}
@@ -49,7 +50,7 @@ void print_blc(struct term *term)
printf("0");
break;
default:
- fprintf(stderr, "invalid type %d\n", term->type);
+ fatal("invalid type %d\n", term->type);
}
}
diff --git a/src/term.c b/src/term.c
index 7ab1a79..50333c7 100644
--- a/src/term.c
+++ b/src/term.c
@@ -6,14 +6,13 @@
#include <string.h>
#include <term.h>
+#include <log.h>
struct term *new_term(term_type type)
{
struct term *term = malloc(sizeof(*term));
- if (!term) {
- fprintf(stderr, "Out of memory!\n");
- abort();
- }
+ if (!term)
+ fatal("Out of memory!\n");
term->type = type;
return term;
}
diff --git a/src/tree.c b/src/tree.c
index 182e1f8..87219ae 100644
--- a/src/tree.c
+++ b/src/tree.c
@@ -10,6 +10,7 @@
#include <string.h>
#include <stdlib.h>
+#include <log.h>
#include <pqueue.h>
#include <tree.h>
@@ -112,8 +113,7 @@ static struct tree *build_tree(struct term *term, void **set)
tree->size = term->u.var.index;
break;
default:
- fprintf(stderr, "invalid type %d\n", term->type);
- return 0;
+ fatal("invalid type %d\n", term->type);
}
if (tree->size < 10) // not suitable for deduplication
@@ -153,8 +153,8 @@ static struct tree *clone_tree_root(struct tree *tree)
new->u.var.index = tree->u.var.index;
break;
default:
- fprintf(stderr, "invalid type %d\n", tree->type);
free(new);
+ fatal("invalid type %d\n", tree->type);
return 0;
}
@@ -176,7 +176,7 @@ static void invalidate_tree(struct tree *tree, int duplication_count)
case VAR:
break;
default:
- fprintf(stderr, "invalid type %d\n", tree->type);
+ fatal("invalid type %d\n", tree->type);
}
}
@@ -195,7 +195,7 @@ static void free_tree(struct tree *tree, int ref_only)
case REF:
break;
default:
- fprintf(stderr, "invalid type %d\n", tree->type);
+ fatal("invalid type %d\n", tree->type);
return;
}
@@ -216,7 +216,7 @@ static void ref_invalidated_tree(struct tree *tree)
case VAR:
break;
default:
- fprintf(stderr, "invalid type %d\n", tree->type);
+ fatal("invalid type %d\n", tree->type);
}
if (tree->state != INVALIDATED_TREE &&
tree->state != VALIDATED_TREE) { // is reffed
@@ -238,12 +238,15 @@ static int cmp_pri(pqueue_pri_t next, pqueue_pri_t curr)
struct list *tree_merge_duplicates(struct term *term)
{
+ debug("building the merkle tree and deduplication set\n");
+
void *set = 0;
build_tree(term, &set);
if (!set)
return 0;
- // construct priority list while deleting set
+ // construct priority queue while deleting set
+ debug("constructing priority queue\n");
struct pqueue *prioritized = pqueue_init(2 << 15, cmp_pri, get_pri);
while (set) {
struct set_element *element = *(struct set_element **)set;
@@ -259,6 +262,7 @@ struct list *tree_merge_duplicates(struct term *term)
struct list *longest = pqueue_pop(prioritized);
final = list_add(final, longest->data);
+ debug("iterating priority queue, invalidating duplicates\n");
struct list *iterator;
while ((iterator = pqueue_pop(prioritized))) {
struct tree *head = iterator->data;
@@ -292,6 +296,7 @@ struct list *tree_merge_duplicates(struct term *term)
}
// destroy invalidated list and replace reffed subtrees
+ debug("replacing invalidated trees with references\n");
iterator = invalidated;
while (iterator) {
ref_invalidated_tree(iterator->data);
@@ -308,6 +313,8 @@ struct list *tree_merge_duplicates(struct term *term)
void tree_destroy(struct list *table)
{
+ return;
+ debug("freeing %d tree elements\n", table->val);
struct list *iterator = table;
while (iterator) {
free_tree(iterator->data, 0);